(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3185],{62166:function(_,a,e){Promise.resolve().then(e.t.bind(e,27820,23)),Promise.resolve().then(e.t.bind(e,3422,23)),Promise.resolve().then(e.t.bind(e,58877,23))},58877:function(){},27820:function(_){_.exports={style:{fontFamily:"'__Noto_Sans_f36179', '__Noto_Sans_Fallback_f36179'",fontStyle:"normal"},className:"__className_f36179",variable:"__variable_f36179"}},3422:function(_){_.exports={style:{fontFamily:"'__Noto_Sans_Arabic_702545', '__Noto_Sans_Arabic_Fallback_702545'",fontStyle:"normal"},className:"__className_702545",variable:"__variable_702545"}}},function(_){_.O(0,[72,404,2971,7023,1744],function(){return _(_.s=62166)}),_N_E=_.O()}]);