# Generated by Django 5.0.6 on 2024-07-13 16:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0052_alter_searchmodelconfig_bi_encoder_docs_encode_config_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="agent",
            name="style_color",
            field=models.CharField(
                choices=[
                    ("blue", "Blue"),
                    ("green", "Green"),
                    ("red", "Red"),
                    ("yellow", "Yellow"),
                    ("orange", "Orange"),
                    ("purple", "Purple"),
                    ("pink", "Pink"),
                    ("teal", "Teal"),
                    ("cyan", "Cyan"),
                    ("lime", "Lime"),
                    ("indigo", "Indigo"),
                    ("fuschia", "Fuschia"),
                    ("rose", "Rose"),
                    ("sky", "Sky"),
                    ("amber", "Amber"),
                    ("emerald", "Emerald"),
                ],
                default="blue",
                max_length=200,
            ),
        ),
        migrations.AddField(
            model_name="agent",
            name="style_icon",
            field=models.CharField(
                choices=[
                    ("Lightbulb", "Lighbulb"),
                    ("Health", "Health"),
                    ("Robot", "Robot"),
                    ("Aperture", "Aperture"),
                    ("GraduationCap", "Graduation Cap"),
                    ("Jeep", "Jeep"),
                    ("Island", "Island"),
                    ("MathOperations", "Math Operations"),
                    ("Asclepius", "Asclepius"),
                    ("Couch", "Couch"),
                    ("Code", "Code"),
                    ("Atom", "Atom"),
                    ("ClockCounterClockwise", "Clock Counter Clockwise"),
                ],
                default="Lightbulb",
                max_length=200,
            ),
        ),
    ]
