# Generated by Django 4.2.5 on 2023-11-09 08:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0013_subscription"),
    ]

    operations = [
        migrations.AlterField(
            model_name="googleuser",
            name="picture",
            field=models.CharField(default=None, max_length=200, null=True),
        ),
    ]
